@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2013 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2014 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,7 @@
 #include <syslog.h>
 #include <unistd.h>
 
+#include "config.h"
 #include "arp.h"
 #include "ipv4.h"
 #include "common.h"
@@ -94,9 +95,9 @@ arp_failure(struct interface *ifp)
 	/* If we failed without a magic cookie then we need to try
 	 * and defend our IPv4LL address. */
 	if ((state->offer != NULL &&
-		state->offer->cookie != htonl(MAGIC_COOKIE)) ||
+	    state->offer->cookie != htonl(MAGIC_COOKIE)) ||
 	    (state->new != NULL &&
-		state->new->cookie != htonl(MAGIC_COOKIE)))
+	    state->new->cookie != htonl(MAGIC_COOKIE)))
 	{
 		ipv4ll_handle_failure(ifp);
 		return;
@@ -105,12 +106,12 @@ arp_failure(struct interface *ifp)
 	unlink(state->leasefile);
 	if (!state->lease.frominfo)
 		dhcp_decline(ifp);
-	dhcp_close(ifp);
-	eloop_timeout_delete(NULL, ifp);
+	eloop_timeout_delete(ifp->ctx->eloop, NULL, ifp);
 	if (state->lease.frominfo)
 		start_interface(ifp);
 	else
-		eloop_timeout_add_sec(DHCP_ARP_FAIL, start_interface, ifp);
+		eloop_timeout_add_sec(ifp->ctx->eloop,
+		    DHCP_ARP_FAIL, start_interface, ifp);
 }
 
 static void
@@ -127,6 +128,7 @@ arp_packet(void *arg)
 	struct if_options *opts = ifp->options;
 	const char *hwaddr;
 	struct in_addr ina;
+	char hwbuf[HWADDR_LEN * 3];
 
 	state = D_STATE(ifp);
 	state->fail.s_addr = 0;
@@ -167,41 +169,52 @@ arp_packet(void *arg)
 		if (state->arping_index &&
 		    state->arping_index <= opts->arping_len &&
 		    (reply_s == opts->arping[state->arping_index - 1] ||
-			(reply_s == 0 &&
-			    reply_t == opts->arping[state->arping_index - 1])))
+		    (reply_s == 0 &&
+		    reply_t == opts->arping[state->arping_index - 1])))
 		{
 			ina.s_addr = reply_s;
 			hwaddr = hwaddr_ntoa((unsigned char *)hw_s,
-			    (size_t)ar.ar_hln);
+			    (size_t)ar.ar_hln, hwbuf, sizeof(hwbuf));
 			syslog(LOG_INFO,
 			    "%s: found %s on hardware address %s",
 			    ifp->name, inet_ntoa(ina), hwaddr);
 			if (select_profile(ifp, hwaddr) == -1 &&
-			    errno == ENOENT)
-				select_profile(ifp, inet_ntoa(ina));
+			    select_profile(ifp, inet_ntoa(ina)) == -1)
+			{
+				state->probes = 0;
+				/* We didn't find a profile for this
+				 * address or hwaddr, so move to the next
+				 * arping profile */
+				if (state->arping_index <
+				    ifp->options->arping_len)
+				{
+					arp_probe(ifp);
+					return;
+				}
+			}
 			dhcp_close(ifp);
-			eloop_timeout_delete(NULL, ifp);
+			eloop_timeout_delete(ifp->ctx->eloop, NULL, ifp);
 			start_interface(ifp);
 			return;
 		}
 
-		/* Check for conflict */
+		/* RFC 2131 3.1.5, Client-server interaction
+		 * RFC 3927 2.2.1, Probe Conflict Detection */
 		if (state->offer &&
 		    (reply_s == state->offer->yiaddr ||
-			(reply_s == 0 && reply_t == state->offer->yiaddr)))
+		    (reply_s == 0 && reply_t == state->offer->yiaddr)))
 			state->fail.s_addr = state->offer->yiaddr;
 
-		/* Handle IPv4LL conflicts */
+		/* RFC 3927 2.5, Conflict Defense */
 		if (IN_LINKLOCAL(htonl(state->addr.s_addr)) &&
-		    (reply_s == state->addr.s_addr ||
-			(reply_s == 0 && reply_t == state->addr.s_addr)))
+		    reply_s == state->addr.s_addr)
 			state->fail.s_addr = state->addr.s_addr;
 
 		if (state->fail.s_addr) {
 			syslog(LOG_ERR, "%s: hardware address %s claims %s",
 			    ifp->name,
 			    hwaddr_ntoa((unsigned char *)hw_s,
-				(size_t)ar.ar_hln),
+				(size_t)ar.ar_hln, hwbuf, sizeof(hwbuf)),
 			    inet_ntoa(state->fail));
 			errno = EEXIST;
 			arp_failure(ifp);
@@ -225,12 +238,13 @@ arp_announce(void *arg)
 			syslog(LOG_ERR, "%s: %s: %m", __func__, ifp->name);
 			return;
 		}
-		eloop_event_add(state->arp_fd, arp_packet, ifp);
+		eloop_event_add(ifp->ctx->eloop,
+		    state->arp_fd, arp_packet, ifp);
 	}
 	if (++state->claims < ANNOUNCE_NUM)
 		syslog(LOG_DEBUG,
 		    "%s: sending ARP announce (%d of %d), "
-		    "next in %d.00 seconds",
+		    "next in %d.0 seconds",
 		    ifp->name, state->claims, ANNOUNCE_NUM, ANNOUNCE_WAIT);
 	else
 		syslog(LOG_DEBUG,
@@ -240,10 +254,14 @@ arp_announce(void *arg)
 		state->new->yiaddr, state->new->yiaddr) == -1)
 		syslog(LOG_ERR, "send_arp: %m");
 	if (state->claims < ANNOUNCE_NUM) {
-		eloop_timeout_add_sec(ANNOUNCE_WAIT, arp_announce, ifp);
+		eloop_timeout_add_sec(ifp->ctx->eloop,
+		    ANNOUNCE_WAIT, arp_announce, ifp);
 		return;
 	}
 	if (state->new->cookie != htonl(MAGIC_COOKIE)) {
+		/* Check if doing DHCP */
+		if (!(ifp->options->options & DHCPCD_DHCP))
+			return;
 		/* We should pretend to be at the end
 		 * of the DHCP negotation cycle unless we rebooted */
 		if (state->interval != 0)
@@ -253,9 +271,9 @@ arp_announce(void *arg)
 		tv.tv_sec = state->interval - DHCP_RAND_MIN;
 		tv.tv_usec = arc4random() % (DHCP_RAND_MAX_U - DHCP_RAND_MIN_U);
 		timernorm(&tv);
-		eloop_timeout_add_tv(&tv, dhcp_discover, ifp);
+		eloop_timeout_add_tv(ifp->ctx->eloop, &tv, dhcp_discover, ifp);
 	} else {
-		eloop_event_delete(state->arp_fd);
+		eloop_event_delete(ifp->ctx->eloop, state->arp_fd);
 		close(state->arp_fd);
 		state->arp_fd = -1;
 	}
@@ -276,7 +294,8 @@ arp_probe(void *arg)
 			syslog(LOG_ERR, "%s: %s: %m", __func__, ifp->name);
 			return;
 		}
-		eloop_event_add(state->arp_fd, arp_packet, ifp);
+		eloop_event_add(ifp->ctx->eloop,
+		    state->arp_fd, arp_packet, ifp);
 	}
 
 	if (state->arping_index < ifp->options->arping_len) {
@@ -302,21 +321,24 @@ arp_probe(void *arg)
 		tv.tv_sec = PROBE_MIN;
 		tv.tv_usec = arc4random() % (PROBE_MAX_U - PROBE_MIN_U);
 		timernorm(&tv);
-		eloop_timeout_add_tv(&tv, arp_probe, ifp);
+		eloop_timeout_add_tv(ifp->ctx->eloop, &tv, arp_probe, ifp);
 	} else {
 		tv.tv_sec = ANNOUNCE_WAIT;
 		tv.tv_usec = 0;
 		if (arping) {
 			state->probes = 0;
 			if (++state->arping_index < ifp->options->arping_len)
-				eloop_timeout_add_tv(&tv, arp_probe, ifp);
+				eloop_timeout_add_tv(ifp->ctx->eloop,
+				    &tv, arp_probe, ifp);
 			else
-				eloop_timeout_add_tv(&tv, start_interface, ifp);
+				eloop_timeout_add_tv(ifp->ctx->eloop,
+				    &tv, start_interface, ifp);
 		} else
-			eloop_timeout_add_tv(&tv, dhcp_bind, ifp);
+			eloop_timeout_add_tv(ifp->ctx->eloop,
+			    &tv, dhcp_bind, ifp);
 	}
 	syslog(LOG_DEBUG,
-	    "%s: sending ARP probe (%d of %d), next in %0.2f seconds",
+	    "%s: sending ARP probe (%d of %d), next in %0.1f seconds",
 	    ifp->name, state->probes ? state->probes : PROBE_NUM, PROBE_NUM,
 	    timeval_to_double(&tv));
 	if (arp_send(ifp, ARPOP_REQUEST, 0, addr.s_addr) == -1)
@@ -332,3 +354,19 @@ arp_start(struct interface *ifp)
 	state->arping_index = 0;
 	arp_probe(ifp);
 }
+
+void
+arp_close(struct interface *ifp)
+{
+	struct dhcp_state *state = D_STATE(ifp);
+
+	if (state == NULL)
+		return;
+
+	if (state->arp_fd != -1) {
+		eloop_event_delete(ifp->ctx->eloop, state->arp_fd);
+		close(state->arp_fd);
+		state->arp_fd = -1;
+	}
+}
+
