@@ -1,6 +1,6 @@
-/* 
+/*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2012 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2014 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -60,78 +60,27 @@
 #  define _PATH_DEVNULL "/dev/null"
 #endif
 
-int clock_monotonic;
-static char *lbuf;
-static size_t lbuf_len;
-#ifdef DEBUG_MEMORY
-static char lbuf_set;
-#endif
-
-#ifdef DEBUG_MEMORY
-static void
-free_lbuf(void)
-{
-	free(lbuf);
-	lbuf = NULL;
-}
-#endif
-
-/* Handy routine to read very long lines in text files.
- * This means we read the whole line and avoid any nasty buffer overflows.
- * We strip leading space and avoid comment lines, making the code that calls
- * us smaller.
- * As we don't use threads, this API is clean too. */
-char *
-get_line(FILE * __restrict fp)
+const char *
+get_hostname(char *buf, size_t buflen, int short_hostname)
 {
 	char *p;
-	ssize_t bytes;
 
-#ifdef DEBUG_MEMORY
-	if (lbuf_set == 0) {
-		atexit(free_lbuf);
-		lbuf_set = 1;
+	if (gethostname(buf, buflen) != 0)
+		return NULL;
+	buf[buflen - 1] = '\0';
+	if (strcmp(buf, "(none)") == 0 ||
+	    strcmp(buf, "localhost") == 0 ||
+	    strncmp(buf, "localhost.", strlen("localhost.")) == 0 ||
+	    buf[0] == '.')
+		return NULL;
+
+	if (short_hostname) {
+		p = strchr(buf, '.');
+		if (p)
+			*p = '\0';
 	}
-#endif
 
-	do {
-		bytes = getline(&lbuf, &lbuf_len, fp);
-		if (bytes == -1)
-			return NULL;
-		for (p = lbuf; *p == ' ' || *p == '\t'; p++)
-			;
-	} while (*p == '\0' || *p == '\n' || *p == '#' || *p == ';');
-	if (lbuf[--bytes] == '\n')
-		lbuf[bytes] = '\0';
-	return p;
-}
-
-int
-set_cloexec(int fd)
-{
-	int flags;
-
-	if ((flags = fcntl(fd, F_GETFD, 0)) == -1 ||
-	    fcntl(fd, F_SETFD, flags | FD_CLOEXEC) == -1)
-	{
-		syslog(LOG_ERR, "fcntl: %m");
-		return -1;
-	}
-	return 0;
-}
-
-int
-set_nonblock(int fd)
-{
-	int flags;
-
-	if ((flags = fcntl(fd, F_GETFL, 0)) == -1 ||
-	    fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1)
-	{
-		syslog(LOG_ERR, "fcntl: %m");
-		return -1;
-	}
-	return 0;
+	return buf;
 }
 
 /* Handy function to get the time.
@@ -143,24 +92,13 @@ set_nonblock(int fd)
 int
 get_monotonic(struct timeval *tp)
 {
-	static int posix_clock_set = 0;
 #if defined(_POSIX_MONOTONIC_CLOCK) && defined(CLOCK_MONOTONIC)
 	struct timespec ts;
-	static clockid_t posix_clock;
 
-	if (!posix_clock_set) {
-		if (clock_gettime(CLOCK_MONOTONIC, &ts) == 0) {
-			posix_clock = CLOCK_MONOTONIC;
-			clock_monotonic = posix_clock_set = 1;
-		}
-	}
-
-	if (clock_monotonic) {
-		if (clock_gettime(posix_clock, &ts) == 0) {
-			tp->tv_sec = ts.tv_sec;
-			tp->tv_usec = ts.tv_nsec / 1000;
-			return 0;
-		}
+	if (clock_gettime(CLOCK_MONOTONIC, &ts) == 0) {
+		tp->tv_sec = ts.tv_sec;
+		tp->tv_usec = ts.tv_nsec / 1000;
+		return 0;
 	}
 #elif defined(__APPLE__)
 #define NSEC_PER_SEC 1000000000
@@ -192,11 +130,13 @@ get_monotonic(struct timeval *tp)
 	}
 #endif
 
+#if 0
 	/* Something above failed, so fall back to gettimeofday */
 	if (!posix_clock_set) {
 		syslog(LOG_WARNING, NO_MONOTONIC);
 		posix_clock_set = 1;
 	}
+#endif
 	return gettimeofday(tp, NULL);
 }
 
@@ -207,7 +147,11 @@ setvar(char ***e, const char *prefix, const char *var, const char *value)
 
 	if (prefix)
 		len += strlen(prefix) + 1;
-	**e = xmalloc(len);
+	**e = malloc(len);
+	if (**e == NULL) {
+		syslog(LOG_ERR, "%s: %m", __func__);
+		return -1;
+	}
 	if (prefix)
 		snprintf(**e, len, "%s_%s=%s", prefix, var, value);
 	else
@@ -236,72 +180,3 @@ uptime(void)
 	return tv.tv_sec;
 }
 
-int
-writepid(int fd, pid_t pid)
-{
-	char spid[16];
-	ssize_t len;
-
-	if (ftruncate(fd, (off_t)0) == -1)
-		return -1;
-	snprintf(spid, sizeof(spid), "%u\n", pid);
-	len = pwrite(fd, spid, strlen(spid), (off_t)0);
-	if (len != (ssize_t)strlen(spid))
-		return -1;
-	return 0;
-}
-
-#ifndef xmalloc
-void *
-xmalloc(size_t s)
-{
-	void *value = malloc(s);
-
-	if (value != NULL)
-		return value;
-	syslog(LOG_ERR, "memory exhausted (xalloc %zu bytes)", s);
-	exit (EXIT_FAILURE);
-	/* NOTREACHED */
-}
-#endif
-
-#ifndef xrealloc
-void *
-xrealloc(void *ptr, size_t s)
-{
-	void *value = realloc(ptr, s);
-
-	if (value != NULL)
-		return value;
-	syslog(LOG_ERR, "memory exhausted (xrealloc %zu bytes)", s);
-	exit(EXIT_FAILURE);
-	/* NOTREACHED */
-}
-#endif
-
-#ifndef xstrdup
-char *
-xstrdup(const char *str)
-{
-	char *value;
-
-	if (str == NULL)
-		return NULL;
-
-	if ((value = strdup(str)) != NULL)
-		return value;
-
-	syslog(LOG_ERR, "memory exhausted (xstrdup)");
-	exit(EXIT_FAILURE);
-	/* NOTREACHED */
-}
-#endif
-
-void *
-xzalloc(size_t s)
-{
-	void *value = xmalloc(s);
-
-	memset(value, 0, s);
-	return value;
-}
