@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2012 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2014 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -31,15 +31,16 @@
 #include "dhcpcd.h"
 
 struct fd_list {
+	struct fd_list *next;
+	struct dhcpcd_ctx *ctx;
 	int fd;
 	int listener;
-	struct fd_list *next;
 };
-extern struct fd_list *control_fds;
 
-int control_start(void);
-int control_stop(void);
-int control_open(void);
-int control_send(int, char * const *);
+int control_start(struct dhcpcd_ctx *, const char *);
+int control_stop(struct dhcpcd_ctx *);
+int control_open(struct dhcpcd_ctx *, const char *);
+int control_send(struct dhcpcd_ctx *, int, char * const *);
+void control_close(struct dhcpcd_ctx *ctx);
 
 #endif
