@@ -250,13 +250,26 @@ discover_interfaces(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 #endif
 #ifdef AF_LINK
 	const struct sockaddr_dl *sdl;
+#ifdef SIOCGIFPRIORITY
+	struct ifreq ifr;
+	int s_inet;
+#endif
 #ifdef IFLR_ACTIVE
 	struct if_laddrreq iflr;
-	int socket_aflink;
+	int s_link;
+#endif
 
-	socket_aflink = socket(AF_LINK, SOCK_DGRAM, 0);
-	if (socket_aflink == -1)
+#ifdef SIOCGIFPRIORITY
+	if ((s_inet = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
 		return NULL;
+#endif
+#ifdef IFLR_ACTIVE
+	if ((s_link = socket(AF_LINK, SOCK_DGRAM, 0)) == -1) {
+#ifdef SIOCGIFPRIORITY
+		close(s_inet);
+#endif
+		return NULL;
+	}
 	memset(&iflr, 0, sizeof(iflr));
 #endif
 #elif AF_PACKET
@@ -265,7 +278,6 @@ discover_interfaces(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 
 	if (getifaddrs(&ifaddrs) == -1)
 		return NULL;
-
 	ifs = malloc(sizeof(*ifs));
 	if (ifs == NULL)
 		return NULL;
@@ -339,8 +351,10 @@ discover_interfaces(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 		}
 
 		ifp = calloc(1, sizeof(*ifp));
-		if (ifp == NULL)
-			return NULL;
+		if (ifp == NULL) {
+			syslog(LOG_ERR, "%s: %m", __func__);
+			break;
+		}
 		ifp->ctx = ctx;
 		strlcpy(ifp->name, p, sizeof(ifp->name));
 		ifp->flags = ifa->ifa_flags;
@@ -378,7 +392,7 @@ discover_interfaces(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 			    MIN(ifa->ifa_addr->sa_len, sizeof(iflr.addr)));
 			iflr.flags = IFLR_PREFIX;
 			iflr.prefixlen = sdl->sdl_alen * NBBY;
-			if (ioctl(socket_aflink, SIOCGLIFADDR, &iflr) == -1 ||
+			if (ioctl(s_link, SIOCGLIFADDR, &iflr) == -1 ||
 			    !(iflr.flags & IFLR_ACTIVE))
 			{
 				free_interface(ifp);
@@ -463,6 +477,13 @@ discover_interfaces(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 			continue;
 		}
 
+#ifdef SIOCGIFPRIORITY
+		/* Respect the interface priority */
+		memset(&ifr, 0, sizeof(ifr));
+		strlcpy(ifr.ifr_name, ifp->name, sizeof(ifr.ifr_name));
+		if (ioctl(s_inet, SIOCGIFPRIORITY, &ifr) == 0)
+			ifp->metric = ifr.ifr_metric;
+#else
 		/* We reserve the 100 range for virtual interfaces, if and when
 		 * we can work them out. */
 		ifp->metric = 200 + ifp->index;
@@ -470,6 +491,7 @@ discover_interfaces(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 			ifp->wireless = 1;
 			ifp->metric += 100;
 		}
+#endif
 
 		TAILQ_INSERT_TAIL(ifs, ifp, next);
 	}
@@ -512,8 +534,11 @@ discover_interfaces(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 
 	freeifaddrs(ifaddrs);
 
+#ifdef SIOCGIFPRIORITY
+	close(s_inet);
+#endif
 #ifdef IFLR_ACTIVE
-	close(socket_aflink);
+	close(s_link);
 #endif
 
 	return ifs;
